/*
    SPDX-FileCopyrightText: 2015 Andreas Cord-Landwehr <cordlandwehr@kde.org>

    SPDX-License-Identifier: LGPL-2.1-or-later
*/

#ifndef LIBSOUND_DEBUG_H
#define LIBSOUND_DEBUG_H

#include <QLoggingCategory>
Q_DECLARE_LOGGING_CATEGORY(LIBSOUND_LOG)

#endif
